function [f,fcmc,N,fS]= INilpolar3(pot, IC, Ir, It, looporder,lambda)
%updated: 21 Feb 2023
% polar coordinate version of the function INil
% see the comments in INil for information about input and output

% Similar to ecmchpolar, only uses the Sym formula that should
%give a minimal surface in indefinite Nil

%The potential, pot, is given as a function handle of (r,t), instead of z, where z=r  exp(it).  

%Ir should be of the form [0, stepsize, n],  where n is the integration is
%over the interval [0, 0+ stepsize*(n+1)].
% It is of the form [t_0, stepsize, m], where t_0 is the initial angle, 
% and the integration is over [t_0,  stepsize*(m+1)]

%the potential pot should be a function handle @(r,t,h), where h is the mean
%curvature.
if nargin<6
   lambda=1;  %can replace with any unit length complex number
end
H=1/2;
A = @(r,t)pot(r,t,H);
%checkinput parses the input for consistency, converts the loops into
%square matrix form and sets the minimum and maximum looporders.
[rpoints,tpoints,bigA,bigIC,minlooporder,maxlooporder,m]=checkinput(A, IC, Ir , It, looporder);
looporders=setlooporders(Ir,minlooporder,maxlooporder);
bsize=m*(2*maxlooporder+1);  %dimensions of the square matrix form of loops
% integrate along the line t=t0+j*stepsize:
Y=zeros(2*rpoints,2*tpoints);  %the minimal surface in Nil
Y2=zeros(2*rpoints,2*tpoints);  %the CMC surface
Y3=zeros(2*rpoints,2*tpoints);  %the normal to the CMC surface
ErrorMatrix1=zeros(rpoints,tpoints);
ErrorMatrix2=zeros(rpoints,tpoints);
for row=1:tpoints
   Phi=realSintegrateA(bigA, eye(bsize), Ir, It(1)+(row-1)*It(2),m);
    for j=1:rpoints
       n=looporders(j);
       k=min([maxlooporder,n]);
       Phimat=loop2SMat(Phi(:, (j-1)*bsize+1:j*bsize),n,k);
       thisPhiloop=bigIC(m*maxlooporder+1:m*maxlooporder+m,m*(maxlooporder-n)+1:m*(maxlooporder+1+n))*Phimat;
       thisPhi=loop2SMat(thisPhiloop,n,k);
       [V,~]=lu2(thisPhi'*thisPhi,m,0.00000001,n);
       U=loop2SMat(V,n,n);
       F=thisPhiloop/U;
       [fval,fcmcval,Nval, UError1,UError2] = sym2(F,lambda);    
       Y(m*(j-1)+1:m*j,m*row-1:m*row)=fval;
       Y2(m*(j-1)+1:m*j,m*row-1:m*row)=fcmcval;
       Y3(m*(j-1)+1:m*j,m*row-1:m*row)=Nval;
       ErrorMatrix1(j,row)=UError1;
       ErrorMatrix2(j,row)=UError2;
    end
   if mod(row,10)==1
        disp(['Row ', num2str(row),  ' Max Error ', num2str(max(ErrorMatrix1(:,row:row))',2),'. Errors: ', num2str(ErrorMatrix1(:,row:row)',2)]); 
        disp(['Row ', num2str(row),  ' Max Error ', num2str(max(ErrorMatrix2(:,row:row))',2),'. Errors: ', num2str(ErrorMatrix2(:,row:row)',2)]); 
        disp(' ');
    end
  clear Phi; 
end
 disp(' ');
disp(['Max error:', num2str(max(max(ErrorMatrix1)),2), '. Mean error: ', num2str(mean(mean(ErrorMatrix1)),2), '. Corner Errors: ']);
  disp(num2str([ErrorMatrix1(1,1), ErrorMatrix1(rpoints,1);ErrorMatrix1(1,tpoints), ErrorMatrix1(rpoints,tpoints)],2));
  disp(['Max error:', num2str(max(max(ErrorMatrix2)),2), '. Mean error: ', num2str(mean(mean(ErrorMatrix2)),2), '. Corner Errors: ']);
  disp(num2str([ErrorMatrix2(1,1), ErrorMatrix2(rpoints,1);ErrorMatrix2(1,tpoints), ErrorMatrix2(rpoints,tpoints)],2));
 N=surfplotdata(Y3);
  fcmc=surfplotdata(Y2);
 fS=fcmc+N;
 f = surfplotdata(Y);
end


      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     %%%%%%%%%%%%%%%%%%%%%%%%%%%% sub functions %%%%%%%%%%%%%%%%

            function [V,p]=lu2(X,m,thresh,n)
   %computes an lu decomposition of X,
   %where X is the small square matrix form of a loop.
   %P is 0 if X is in the big cell
[~,Umat,P]=lu(X(1:m*(n+1),1:m*(2*n+1)),thresh);
p=trace(P)-length(P);
  UmDims=size(Umat);
  V=Umat(UmDims(1)-m+1:UmDims(1),1:m*(2*n+1));
           end
 



    function F = realSintegrateA(A, F0, Ix,t0,m)
    %solves the equation dF/dt = F A, with initial condition F(t0) = F0, with t
%in the specified interval
%The values of F are returned in loops in row form

tempIx3=max(Ix(3),2); 
interval=0:Ix(2):Ix(2)*(tempIx3);

bsize=length(F0);
n =(bsize/m-1)/2; % the order of X in lambda
initF=reshape(F0(m*n+1:m*n+m,:),m*bsize,1);
testfn = @(r,y) reshape(sparse(reshape(y,m,bsize))*exp(1i*t0)*A(r,t0), m*bsize,1);
[~,sol] = ode45(testfn ,interval,initF); 
F=sparse(reshape(sol(1:Ix(3)+1, :).', m,bsize*(Ix(3)+1)));  

end

 function Y = rloopeval(X,r)
% Evaluates a loop at lambda = r.
% Input: X = [x_-n .... x_0 x_1 .... x_n], where each x_i is 2x2
%  r is a number.
%Output: a sparse 2x2 matrix, the sum of x_i r^i
n = (length(X)-2)/4; % the order of X in lambda

Y= X(1:2, 2*n+1:2*n+2);
for count = 1 : n 
    Y = Y +  r^count*X(1:2, 2*n+1+2*count : 2*n+2+ 2*count) +  r^(-count)*X(1:2, 2*n+1-2*count : 2*n + 2 - 2*count);
end
end

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [f,fcmc,N, UError1,UError2] = sym2(X,lambda)
%computes the the Sym formula 
%-2i r  off_diag(A) + diag( -2r A - 2r^2 B +2r^2(A^2))
%where A = (d X/ d r ) X^{-1} evaluated at lambda=r and
% B = (d^2 X/ d^2 r ) X^{-1}   evaluated at r^
%D= diag(sqrt(lambda), 1/sqrt(lambda))
%here r is the loop parameter, and  X should be the untwisted frame.

%Input: X is a 2x2(2n+1) matrix X=[B_1,.......,B_points], where each block
%B_i is itself a 2x2  block diagonal B_i = [s_i, s_i, .. s_i], where s_i is
%the value of f at point i.
%n =(length(Xmat)-2)/8; % the order of X in lambda
%X= Xmat(length(Xmat)/2: length(Xmat)/2 +1,  2*n+1 : 6*n+2); %convert matrix to loop 
n =(length(X)-2)/4; % the order of X in lambda
Xdiff=zeros(2,2);
Xdiff2=zeros(2,2);
r=lambda;
  for j=1:n
      Xdiff=Xdiff+j*r^(j-1)*X(:,2*(n+j)+1:2*(n+j+1));
      Xdiff2=Xdiff2+j*(j-1)*r^(j-2)*X(:,2*(n+j)+1:2*(n+j+1));
  end
  for j=1:n
      Xdiff=Xdiff-j*r^(-j-1)*X(:,2*(n-j)+1:2*(n-j+1));
      Xdiff2=Xdiff2+j*(j+1)*r^(-j-2)*X(:,2*(n-j)+1:2*(n-j+1));
  end
Xval=rloopeval(X,r);
A=Xdiff/Xval;
B=Xdiff2/Xval;
T1=-(2*1i)*r*A; 
T2=-2*r*(A+r*B-r*A*A);
f=[T2(1,1), T1(1,2); T1(2,1), T2(2,2)];  %the off-diagonal and diagonal parts of T1 and T2
UError1=0;
UError2=0;
if norm(T1)>0
  UError1=norm(T1'+T1)/norm(T1); %check to see if the matrix is in U(2)
end
%T2=1i*T2;
if norm(T2)>0
  UError2=norm(T2'+T2)/norm(T2); %check to see if the matrix is in U(2)
end
  fcmc=-(2*1i*lambda)*A-(1/2)*[1i,0;0,-1i];
  N=(1/2)*Xval*[1i,0;0,-1i]/(Xval);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Y = surfplotdata(f)
% Uses the basis e_3 = (1/2)[i 0  ; 0 -i] etc for su(2)
fheight=length(f(:,1:1));
xpoints=fheight/2;
fwidth=length(f(1:1,:));
ypoints=fwidth/2;
Y=zeros(3*xpoints, ypoints);
for k=1:xpoints
    for j=1:ypoints    
        Y(k,j) = -2*imag(f(2*k-1, 2*j));
        Y(xpoints + k, j) = 2*real(f(2*k-1, 2*j));     
        Y(2*xpoints+k, j) =2*imag(f(2*k-1, 2*j-1));   
    end
end
surf(Y(1:xpoints, :), Y(xpoints+1: 2*xpoints, :),  Y(2*xpoints+1: 3*xpoints, :),'Clipping','off','EdgeAlpha',0.4,'FaceAlpha',1,'FaceLighting','gouraud','EdgeLighting','gouraud','DiffuseStrength',0.8,'EdgeColor','white','FaceColor',[0.05,0.55,0.85])
 daspect([1 1 1])
 grid off
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function looporders=setlooporders(Ir,minlooporder,maxlooporder)

looporders=maxlooporder*ones(1,Ir(3)+1);
if maxlooporder>minlooporder
  rstepsize=(maxlooporder-minlooporder)/Ir(3);
  looporders=min(looporders,ceil((minlooporder/rstepsize:1:maxlooporder/rstepsize)*rstepsize));  
end
end


function Xmat = loop2SMat(X,n,k)
%Converts a loop into a big square matrix 
%Input:  a matrix X=[x_-nold .... x_0 x_1 .... x_nold] representing a Laurent
%polynomial loop of order nold
%The input loop does not need to be the same order as the output loop

%x_-nold lambda^-nold + ... + x_0 + x_nold lambda^nold, where each x_i is an mxm matrix 
%Output:  Xmat is a sparse m(4n+1) x m(4n+1) square matrix, which looks something
%like  [x_0 ... x_n 0 0 ... 0;
%       x_-1 x_0 ....x_n 0 ...;
%       x_-2 x_-1 x_0 .......; ..... etc
% Multiplying  Xmat with Ymat corresponds to multiplying the loops X and Y.
Xdims=size(X);
m=Xdims(1);
nold =(Xdims(2)/m-1)/2; % the order of the input loop X in lambda
k=min(nold,k);
Xmat=zeros(m*(2*n+1));
Xmat(m*n+1:m*(n+1),m*(n-k)+1:m*(n+1+k))=full(X(1:m, m*(nold-k)+1: m*(nold+k+1)));
for j=0:n-1
    Xmat(m*j+1:m*j+m,1:m*(n+1+j))=Xmat(m*n+1:m*(n+1),m*(n-j)+1:m*(2*n+1));
    Xmat(m*(2*n-j)+1:m*(2*n-j+1),m*(n-j)+1:m*(2*n+1))=Xmat(m*n+1:m*(n+1),1:m*(n+1+j));
end
Xmat=sparse(Xmat);
end


function [rpoints,tpoints,bigA,bigIC,minlooporder,maxlooporder,m]=checkinput(A, IC, Ir , It, looporder)

%Checks the input data for consistency with expected input.
%Converts the intial data, in terms of simple untwisted loops, into 
%untwisted loops represented by big square matrices, expanded up to order
%'looporder' or the maximum order of the other data in the loop parameter,
%whichever is the greater.

% Output:  Each of bigAp and bigAm is a handle for a function of the form  = @(t) M(t)
% where M is a  2(4n+1) x 2(4n+1) square matrix valued function of t
% which looks something like 
%      [x_0 ... x_n 0 0 ... 0;
%       x_-1 x_0 ....x_n 0 ...;
%       x_-2 x_-1 x_0 .......; ..... etc,  each x_i a 2x2 matrix

% bigFp0 and bigFm0 are square matrices of initial values, of the same size 
%and similar structure to A.
assert((Ir(3)>0),'Ir(3) must be positive');
assert((It(3)>0),'It(3) must be positive');
rpoints=1+Ir(3); tpoints=1+It(3);

ICDims=size(IC); 
ADims=size(A(Ir(1),It(1)));
m=ICDims(1);
assert((ICDims(1)==2)&(ADims(1)==2),'All input loops must be matrices with two rows');

ICorder = (ICDims(2)-m)/(2*m);  
AOrder = (ADims(2)-m)/(2*m);  
assert(norm(floor([ICorder, AOrder])-[ICorder, AOrder])==0,...
    'Number of columns for input matrices and function handles are not correct for loops');
loDims=size(looporder);
assert((loDims(2)<=2)&(norm(floor(looporder)-looporder)==0)&(norm(looporder-abs(looporder))==0),...
    'looporder must be a non-negative integer or a 1x2 non-negative integer matrix ');
if loDims(2)==2
    maxlooporder=max([looporder(1),ICorder,AOrder]);
    minlooporder=looporder(2);
else
  maxlooporder=max([looporder,ICorder,AOrder]); 
  minlooporder=max([floor(maxlooporder/4),ICorder]);  
end
%convert the loops into square matrix form:
bigA=@(r,t)loop2SMat(A(r,t),maxlooporder,AOrder);
bigIC= loop2SMat(IC,maxlooporder,ICorder);
 end

%%%%%%%%%%%%%%%%%%%%%%%%%%