function plotall(N,f,g,m,n)
%plots the three surfaces N, f and fcmc all with a colormap determined by N
%N is assumed to take values in the unit sphere, and a different
%colormap is used for the upper hemisphere from the lower

%Input: N, f and g are all matrices of the same dimensions, of the form:
%     N= [X; Y; Z]
% where X, Y and Z are matrices containing the x y and z coodinate values
% for a surface.
% m and n are the number of coordinate curves to plot for  the
% x and y coordinates
if nargin<4 
    m=10;
    n=10;
end

%First make the mesh denser:
N=interpf(N,3);
f=interpf(f,3);
g=interpf(g,3);
P=getcmap(N); %create the colormap
%Plot the surfaces:
figure(); plotf2zb(N,P,m,n);
hold on; sphereplot([0,0,0],0.98,1,[0.9,0.9,1]);
figure(); plotf2zb(f,P,m,n); 
figure(); plotf2zb(g,P,m,n);

end

function C=getcmap(N)
%input: N=[X;Y;Z] grid values for a surface
%output: a color map C consisting of zeros and 1's indicating where the Z
%value of N is positive or not
[r,~]=size(N);
C=sign(N(2*r/3+1:r,:));
C=C+(C==0); %replace zeros with ones
C=(1/2)*(C+1); %convert to binary matrix
end


function  plotf2zb(f,P,numx,numy,linewidth,scolor1,edgecolor1,scolor2,edgecolor2)
%plots the surface f=[X;Y;Z] where each of X,Y and Z is m x n
% P is an m x n matrix indicating two zones on the surface
%P(i,j) non-zero means zone one, otherwise zone 2.

fdims=size(f);  xpoints=fdims(1)/3; ypoints=fdims(2);
if nargin<4
    numy=min(10,xpoints);
end
if nargin<3
    numx=min(10,ypoints);
end
if nargin<5
    linewidth=1;
end
if nargin<6
    scolor1=[0.95,0.95,0.95];
end
if nargin<7
    edgecolor1=[0.7  0.1,0.1];
end
if nargin<8
    scolor2=[0.3 0.42 0.3];
end
if nargin<9
    edgecolor2=  [0.95 0.95 0.95];  
end
P=P~=0;  % normalize P to logical array

colormap([scolor1;edgecolor1;scolor2;edgecolor2]);
C=zeros(xpoints,ypoints);
if numx>0
  if numx==1
    xcurves=1;
  else
    xcurves=floor(linspace(1,ypoints,numx));
  end
  for j=1:floor(numx/2)
      C(1:xpoints,xcurves(j):xcurves(j)+linewidth-1)=1;
  end
  for j=floor(numx/2)+1:numx
      C(1:xpoints,xcurves(j)-linewidth+1:xcurves(j))=1;
  end
end

if numy>0
  if numy==1
    ycurves=1;
  else
       ycurves=floor(linspace(1,xpoints,numy));
  end 
  for j=1:floor(numy/2)
      C(ycurves(j):ycurves(j)+linewidth-1,:)=1;
  end
  for j=floor(numy/2)+1:numy
      C(ycurves(j)-linewidth+1:ycurves(j),:)=1;
  end
end
 if numel(unique(P))>1
  C=C+2*P;  %shift the colorgrid
  colormap([scolor1;edgecolor1;scolor2;edgecolor2]);
else
   colormap([scolor1;edgecolor1]);
 end

surf(f(1:xpoints, :), f(xpoints+1: 2*xpoints, :),  f(2*xpoints+1: 3*xpoints, :),C, 'Clipping','off','EdgeAlpha',0,'FaceLighting','gouraud',...
    'DiffuseStrength',0.8,'SpecularExponent',20.0,'SpecularStrength',0.2,'AmbientStrength',0.35);
daspect([1,1,1])
  hold off
end




function  g=interpf(f,k)
%smooths the surface f=[X;Y;Z] by interpolating the points with cubic
%splines. k is the number of points to insert between the existing points

fdims=size(f);  xpoints=fdims(1)/3; ypoints=fdims(2);
X=f(1:xpoints,:);
Y=f(xpoints+1:2*xpoints,:);
Z=f(2*xpoints+1:3*xpoints,:);
[A,B]=meshgrid(0:1/(ypoints-1):1,0:1/(xpoints-1):1);
[Aq,Bq]=meshgrid(0:1/(k*(ypoints-1)):1,0:1/(k*(xpoints-1)):1);
Xq=interp2(A,B,X,Aq,Bq,'cubic');
Yq=interp2(A,B,Y,Aq,Bq,'cubic');
Zq=interp2(A,B,Z,Aq,Bq,'cubic');
g=[Xq;Yq;Zq];
end


function sphereplot(c,r,salpha,scolor,ealpha,ecolor)
if nargin<6
    ecolor=[1,1,1];
end
if nargin<5
    ealpha=0;
end
if nargin<4
    scolor=[0.8,0.8,0.8];
end
if nargin<3
    salpha=0.3;
end
  [T1,T2,T3]=sphereshell(c,r);
  surf(T1,T2,T3,'EdgeAlpha',ealpha,'EdgeColor',ecolor,'FaceAlpha',salpha,'FaceLighting','gouraud','FaceColor',scolor)
  daspect([1 1 1])
end

function [T1,T2,T3]=sphereshell(c,r)
t=0:pi/100:2*pi;
s=0:pi/100:pi;

for tcount=1:201
    for scount=1:101
        T1(tcount,scount)=r*cos(t(tcount))*sin(s(scount))+c(1);
        T2(tcount,scount)=r*sin(t(tcount))*sin(s(scount))+c(2);
        T3(tcount,scount)=r*cos(s(scount))+c(3);
    end
end
end



